package net.idscan.android.pdf417scannerexample;

import android.content.Intent;
import android.os.AsyncTask;
import android.os.Bundle;
import android.support.v4.app.FragmentActivity;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;
import android.widget.Toast;

import net.idscan.android.pdf417scanner.PDF417ScanActivity;
import net.idware.android.dlplib.DriverLicense;
import net.idware.android.dlplib.DriverLicenseParserFactory;
import net.idware.android.dlplib.IDriverLicenseParser;
import net.idware.android.dlplib.ParserException;

import java.lang.ref.WeakReference;

public class MainActivity extends FragmentActivity
{
	private final static int SCAN_ACTIVITY_CODE = 0x001;

	/**
	 * Parser's Results.
	 *
	 * @author andrew0x1@gmail.com
	 */
	private interface IResultHandler
	{
		void onDriverLicense(DriverLicense result);
		void onError(ParserException error);
	}

	/**
	 * We have to call our web services. In our case we will do in in
	 * a separate thread. We will use AsyncTask
	 *
	 * @author andrew0x1@gmail.com
	 */
	private final static class _ParsingAction extends AsyncTask<String, Integer, DriverLicense>
	{
		private final WeakReference<IResultHandler> _handler;
		private volatile ParserException _err = null;

		_ParsingAction(IResultHandler handler)
		{
			if(handler == null)
				throw new NullPointerException("Parameter 'handler' can't be null.");

			_handler = new WeakReference<>(handler);
		}

		@Override
		protected DriverLicense doInBackground(String... params)
		{
			try
			{
				//We need to create an instance for parser and create
				//a web service call.In order to do that we need to
				// add <uses-permission android:name="android.permission.INTERNET" />
				// in AndroidManifest.xml.
				IDriverLicenseParser parser = DriverLicenseParserFactory.createPaser("[Your AuthKey]");
				return parser.parse(params[0]);
			}
			catch(ParserException ex)
			{
				_err = ex;
				return null;
			}
		}

		@Override
		protected void onPostExecute(DriverLicense result)
		{
			//Processing results in the main UI thread .
			IResultHandler h = _handler.get();
			if(h != null)
			{
				if(result != null)
					h.onDriverLicense(result);
				else if(_err != null)
					h.onError(_err);
			}
		}
	}

	/** UI elements. */
	private TextView _tv_result;
	private Button _btn_parse;

	/** Current async parsing. */
	private _ParsingAction _current_action = null;

	/** License key for scanner. */
	private static final String _key = "SK11YT+TPHT8h4Ab41kH+x5Nhz3u2rdfUlCG6vogOjfsH+8c5zzMLSfACVkcYWfjMDbqhWq7eGN8eR+Rj0bblPm1wsD0NfhhfeTIYMtn78OIbjoRqvH1FSMC7Hpyzy1QLI1v53Hns10cqp6ksEQGDcERDW/fwcRROrrSY0c0Bu8=";

	@Override
	protected void onCreate(Bundle savedInstanceState)
	{
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_main);

		_tv_result = (TextView)findViewById(R.id.tv_result);

		Button _btn_scan = (Button)findViewById(R.id.btn_scan);
		_btn_scan.setOnClickListener(new View.OnClickListener()
		{
			@Override
			public void onClick(View view)
			{
				// Default activity.
				//Intent i = new Intent(MainActivity.this, PDF417ScanActivity.class);

				// Custom activity.
				Intent i = new Intent(MainActivity.this, CustomScanActivity.class);

				// Setup license key.
				i.putExtra(PDF417ScanActivity.EXTRA_LICENSE_KEY, _key);
				startActivityForResult(i, SCAN_ACTIVITY_CODE);
			}
		});

		_btn_parse = (Button)findViewById(R.id.btn_parse);
		_btn_parse.setOnClickListener(new View.OnClickListener()
		{
			@Override
			public void onClick(View v)
			{
				_parse();
			}
		});
	}

	@Override
	protected void onDestroy()
	{
		super.onDestroy();
		_cancel();
	}

	@Override
	protected void onResume()
	{
		super.onResume();
		_updateUI();
	}

	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data)
	{
		super.onActivityResult(requestCode, resultCode, data);

		if(requestCode == SCAN_ACTIVITY_CODE)
		{
			if(resultCode == PDF417ScanActivity.RESULT_OK)
			{
				if(data != null)
					_tv_result.setText(data.getStringExtra(PDF417ScanActivity.BARCODE_DATA));
			}
			else if(resultCode == PDF417ScanActivity.ERROR_INVALID_CAMERA_NUMBER)
			{
				_tv_result.setText("Invalid camera number.");
			}
			else if(resultCode == PDF417ScanActivity.ERROR_CAMERA_NOT_AVAILABLE)
			{
				_tv_result.setText("Camera not available.");
			}
			else if(resultCode == PDF417ScanActivity.ERROR_INVALID_CAMERA_ACCESS)
			{
				_tv_result.setText("Invalid camera access.");
			}
			else if(resultCode == PDF417ScanActivity.ERROR_INVALID_LICENSE_KEY)
			{
				_tv_result.setText("Invalid license key.");
			}

			_updateUI();
		}
	}

	private void _parse()
	{
		final String data = _tv_result.getText().toString();
		if(data.length() == 0)
		{
			Toast.makeText(this, "Input field is empty!", Toast.LENGTH_SHORT).show();
			return;
		}

		if(_current_action == null)
		{
			_tv_result.setText("Parsing...");

			_current_action = new _ParsingAction(_handler);
			_updateUI();
			_current_action.execute(data);
		}
	}

	private void _cancel()
	{
		if(_current_action != null)
		{
			_current_action.cancel(false);
			_current_action = null;
			_updateUI();
		}
	}

	private void _updateUI()
	{
		if((_current_action == null) && (_tv_result.getText().length() > 0))
		{
			_btn_parse.setEnabled(true);
		}
		else
		{
			_btn_parse.setEnabled(false);
		}
	}

	private final IResultHandler _handler = new IResultHandler()
	{
		@Override
		public void onDriverLicense(DriverLicense result)
		{
			_current_action = null;
			_updateUI();

			final String address = result.getAddress1();
			if(address != null)
				_tv_result.setText("Address: " + address);
			else
				_tv_result.setText("Address: empty");
		}

		@Override
		public void onError(ParserException error)
		{
			_current_action = null;
			_updateUI();

			_tv_result.setText("Error");

			final String msg = error.getMessage();
			if(msg != null)
				Toast.makeText(MainActivity.this, msg, Toast.LENGTH_SHORT).show();
		}
	};
}
